# -*- coding: utf-8 -*-
"""
Created on Wed Sep 29 23:00:00 2021

@author: Claudiu Stan

Version tracking: curated version of EvAvgCSPADraw.py

This script saves a Matlab file (RunXXXEvDetAvgRaw.mat, where XXX is the run
number) containg the information needed for synchonization of LCLS detector
data with the optical image data:
    - Evt: LCLS event number within the experimental run
    - Sec: Time variable #1 - seconds
    - Nsec: Time variable #2 - nanoseconds
    - Fid: Time variable #3 - LCLS fiducial number
    - PuEnrg: Pulse energy in mJ, as reported by the LCLS gas detector
    - AvgSig: Average signal of the whole CSPAD detector [ADU].
    - AvgSigAtt: Average signal over most of the post-attenuator shadowed region of the CSPAD detector [ADU]
    calculates the *raw* average signal in a CSPAD image, for all events
    
INPUTS:
    - run number, entered from the command line

OUTPUTS:
    - the outputs are in the saved .mat file
    - event numbers and special conditions are reported live during processing,
      such as missing detector data

NOTE 1: All CSPAD data is in the raw format (without any corrections)

Nore 2: The script uses Python 3 and is specific to the LG15 experiment. The
version of Python used in the data analysis environment at SLAC, as well as 
some of the SLAC data analysis packages and data formats could be updated,
which would require modifications to keep the script functional. This script 
was last used (sucessfully) in April 2023.

"""

from psana import * # Load psana libraries for direct calling
import numpy as np # Load numerics package
import scipy.io as sio # Load matlab import/export libraries
import h5py # Load hdf5 access libraries
import matplotlib.pyplot as plt # Load mat-plotting library

# Get run and event number from command line
Run = input("Run number?")
print( "Run number:",str(Run) )

# Mount the XTC run data, define the CSPAD data object
callN = "exp=cxig1515:run="+str(Run)
ds  = DataSource(callN)
cspadDet = Detector('Ds1CsPad',ds.env()) # CSPAD images
gasDet = Detector('FEEGasDetEnergy') # Gas detector pulse energy
print( "Data mounted" )

# Define the output arrays
Xsctr = np.zeros((0), dtype=np.float64)
XsctrAtt = np.zeros((0), dtype=np.float64)
Secs = np.zeros((0), dtype=np.float64)
Nsecs = np.zeros((0), dtype=np.float64)
TimeIdx = np.zeros((0), dtype=np.uint64)
Fids = np.zeros((0), dtype=np.float32)
Evts = np.zeros((0), dtype=np.float32)
Penrg = np.zeros((0), dtype=np.float32)

nevent = 0
for evt in ds.events():
    dta = cspadDet.raw(evt)  # raw unassembled detector data
    dta = np.float64(dta)
    nevent += 1 # Counts all events; CSPAD event count is given by array length
    TimeData = evt.get(EventId) # Get all timing, including timing for shots with missing CSPAD data
    TSecs = TimeData.time()[0]
    TNsecs = TimeData.time()[1]
    TFids = TimeData.fiducials()
    if dta is None:
        print( "No cspad data after Ev:", str(nevent).zfill(5) )
        Scatter = np.nan
        ScatterAtt = np.nan
    elif np.size(dta) != 2296960:
        print( "Wrong cspad data after Ev:", str(nevent).zfill(5) )
        Scatter = np.nan
        ScatterAtt = np.nan
    else:
        # Get the CSPAD average, and the time information
        Scatter = np.mean(dta)
        ScatterAtt = np.mean(dta[[0,1,7,8,15,16,23,24],0:185:1,0:370:1])
    
    # Get pulse energy from gas detector
    gas = gasDet.get(evt)
    if gas is None:
        print( "No gas detector value after Ev:", str(nevent).zfill(5) )
        Tgas = np.nan
    else:
        Tgas = 0.25*(gas.f_11_ENRC()+gas.f_12_ENRC()+gas.f_21_ENRC()+gas.f_22_ENRC())

    # Write data
    Xsctr = np.insert(Xsctr,len(Xsctr),np.float64(Scatter))
    XsctrAtt = np.insert(XsctrAtt,len(XsctrAtt),np.float64(ScatterAtt))
    Secs = np.insert(Secs,len(Secs),np.float64(TSecs))
    Nsecs = np.insert(Nsecs,len(Nsecs),np.float64(TNsecs))
    Fids = np.insert(Fids,len(Fids),np.float32(TFids))
    Evts = np.insert(Evts,len(Evts),np.float32(nevent))
    Penrg = np.insert(Penrg,len(Penrg),np.float32(Tgas))
    # 64-bit time integers for sorting
    TimeIdx = np.insert(TimeIdx,len(TimeIdx),(int(Secs[-1])<<32)|int(Nsecs[-1]))
    
    # Report interation number
    print( nevent )
    # Early exit for testing
    # if nevent>239: break

# Sort output data chronologically
usXsctr = Xsctr
usXsctrAtt = XsctrAtt
usSecs = Secs
usNsecs = Nsecs
usFids = Fids
usEvts = Evts
usPenrg = Penrg

IdxSrt = np.argsort(TimeIdx)

for i in range(len(TimeIdx)):
    Xsctr[i] = usXsctr[IdxSrt[i]]
    XsctrAtt[i] = usXsctrAtt[IdxSrt[i]]
    Secs[i] = usSecs[IdxSrt[i]]
    Nsecs[i] = usNsecs[IdxSrt[i]]
    Fids[i] = usFids[IdxSrt[i]]
    Evts[i] = usEvts[IdxSrt[i]]
    Penrg[i] = usPenrg[IdxSrt[i]]

    
# Save data to Matlab file
OutpDict = {'Evt':Evts, 'Sec':Secs, 'Nsec':Nsecs, 'Fid':Fids, 'AvgSig':Xsctr, 'AvgSigAtt':XsctrAtt, 'PuEnrg':Penrg, 'AllEvts':nevent}
MatFile = "Run"+str(Run).zfill(3)+"EvDetAvgRaw.mat"
sio.savemat(MatFile,OutpDict,oned_as='column')